import { GeoAPIResponse, GeolocationCoordinates } from './GeoApiClient.types.ts'
import { ResolvesCity } from '../../usecases/dependencies/ResolvesCity.ts'
import { ResolvesCoordinates } from '../../usecases/dependencies/ResolvesCoordinates.ts'
import { Logs } from '../../usecases/dependencies/Logs.ts'
import { GraphQLError } from '../../deps.ts'

export class GeoApiClient implements ResolvesCoordinates, ResolvesCity {
  constructor(private readonly endpointUrl: string, private readonly logger: Logs) {
  }

  async resolveCoordinates(
    geolocationId: string,
  ): Promise<GeolocationCoordinates> {
    const response = await this.fetchPlaceDetails(geolocationId)
    const { lat, lon } = response.data?.placeDetails?.geolocation?.properties || {}
    if (lat && lon) {
      return { lat, lon }
    } else {
      throw new GraphQLError(
        `Resolution of lat/lon failed (no data in response for geolocationId=${geolocationId})`,
      )
    }
  }

  async resolveCityName(
    geolocationId: string,
  ): Promise<string> {
    const response = await this.fetchPlaceDetails(geolocationId)
    const { city } = response.data?.placeDetails?.geolocation?.properties || {}
    if (city) {
      return city
    } else {
      throw new GraphQLError(
        `Resolution of city name failed (no data in response for geolocationId=${geolocationId})`,
      )
    }
  }

  private async fetchPlaceDetails(
    geolocationId: string,
  ): Promise<GeoAPIResponse> {
    const graphQLQuery = `{ placeDetails(id: "${geolocationId}") { geolocation } }`
    const response = await fetch(this.endpointUrl, {
      'body': JSON.stringify({ query: graphQLQuery }),
      'headers': {
        'Accept': 'application/graphql-response+json, application/json',
        'Content-Type': 'application/json',
      },
      'method': 'POST',
    })

    const body = await response.text()
    try {
      const data = JSON.parse(body)
      if (Object.hasOwn(data, 'errors')) {
        this.logger.error(data.errors)
        throw new GraphQLError("Can't fetchPlaceDetails for geolocationId " + geolocationId)
      }

      return body as GeoAPIResponse
    } catch (e) {
      this.logger.error(`Response is not JSON: "${body}"`)
      throw e
    }
  }
}
